# This file is part of Online Services Jenkaas.
#
# Copyright 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals


import os


from olsvms import subprocesses


import olsjenkaas


class Repository(object):
    """A facade for local git repositories.

    The command line tool (and its API) are used to reduce the coupling.
    """

    def __init__(self, location):
        self.location = os.path.abspath(location)

    def _run(self, args):
        return subprocesses.run(['git'] + args)

    def tip_commit_id(self):
        with olsjenkaas.working_directory(self.location):
            return self._run(['rev-parse', 'HEAD'])[1].strip()

    def create(self):
        return self._run(['init', self.location])

    def create_from(self, existing):
        return self._run(['clone', existing, self.location])

    def update(self, description, tree_update):
        from olsjenkaas.tests import fixtures
        with olsjenkaas.working_directory(self.location):
            fixtures.build_tree(tree_update)
            self._run(['add', '.'])
            return self._run(['commit', '-m', description])

    def push(self, url, branch='master'):
        with olsjenkaas.working_directory(self.location):
            return self._run(['push', url, branch])

    def delete(self):
        with olsjenkaas.working_directory(self.location):
            return subprocesses.run(['rm', '-fr', self.location])

    def status(self):
        with olsjenkaas.working_directory(self.location):
            ret, out, err = self._run(['status', '--short'])
            return out
