#
# Copyright 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import unittest


from olstests import scenarii


from olsjenkaas import launchpad
from olsjenkaas.tests import fixtures


load_tests = scenarii.load_tests_with_scenarios


class TestProposals(unittest.TestCase):

    scenarios = [('bzr', dict(branch_handler=fixtures.BzrBranchHandler(),
                              proposal_handler=fixtures.BzrProposalHandler())),
                 ('git', dict(branch_handler=fixtures.GitRepositoryHandler(),
                              proposal_handler=fixtures.GitProposalHandler()))]

    def setUp(self):
        super(TestProposals, self).setUp()
        self.lp = fixtures.use_qastaging(self)
        self.branch_handler.set_identity(self)
        self.branch_handler.set_launchpad_access(self)
        fixtures.setup_proposal_target(self)

    def test_no_mps(self):
        self.assertEqual([], list(self.target_lp_branch.landing_candidates))

    def test_create_simple_mp(self):
        proposal = fixtures.create_proposal(self, 'proposal-1')
        self.assertEqual('Needs review', proposal.queue_status)


class TestVotes(unittest.TestCase):

    scenarios = [('bzr', dict(branch_handler=fixtures.BzrBranchHandler(),
                              proposal_handler=fixtures.BzrProposalHandler())),
                 ('git', dict(branch_handler=fixtures.GitRepositoryHandler(),
                              proposal_handler=fixtures.GitProposalHandler()))]

    def setUp(self):
        super(TestVotes, self).setUp()
        self.lp = fixtures.use_qastaging(self)
        self.branch_handler.set_identity(self)
        self.branch_handler.set_launchpad_access(self)
        fixtures.setup_proposal_target(self)
        self.proposal = fixtures.create_proposal(self, 'wip')

    def assertApproved(self):
        approved = launchpad.is_approved(self.proposal, self.target_lp_branch)
        self.assertTrue(approved)

    def assertNotApproved(self):
        approved = launchpad.is_approved(self.proposal, self.target_lp_branch)
        self.assertFalse(approved)

    def test_no_votes(self):
        # No vote means the proposal is not approved
        self.assertNotApproved()

    def test_last_vote_wins(self):
        # Only the last vote is taken into account
        self.proposal.createComment(subject='subject1', vote='Approve',
                                    content='content1')
        self.assertApproved()
        self.proposal.createComment(subject='subject2', vote='Needs Fixing',
                                    content='content1')
        self.assertNotApproved()

    def test_outsider_vote(self):
        # Fake an outsider vote

        # outsiders can't approve
        pass

    def test_pending_vote(self):
        # A pending vote blocks proposal approval
        # FIXME: A better test would be to add a review request for another
        # user -- vila 2017-01-26
        self.assertNotApproved()

    def test_approve_vote(self):
        # One authorized approved vote approves the proposal
        self.proposal.createComment(subject='subject1', vote='Approve',
                                    content='content1')
        self.assertApproved()
