import java.text.*
import jenkins.model.*
import jenkins.security.*
import jenkins.security.s2m.*
import hudson.*
import hudson.model.*
import hudson.slaves.*
import hudson.security.*
import hudson.plugins.openid.*

// 'teams' is a list of users and/or groups that will be given
// most administrator rights in a jenkaas deployment.  
def teams = ['{{jenkaas}.admin.user}']
def build_bots = ['{{jenkaas}.landing.user}']
def admin_bots = ['{{jenkaas}.bot.user}']
def build_people = ['{{jenkaas}.build.user}']
// FIXME: Only one slave is supported ! -- vila 2016-08-22
def slave_name = "{{jenkaas}.slaves}";



def instance = Jenkins.getInstance();

def perms = Permission.getAll();

def strategy = new GlobalMatrixAuthorizationStrategy()

for (team in teams) {
    for (perm in perms) {
        def id = perm.getId()
        def pattern = "Credential"
        def m = id =~ /$pattern/
        if (m || id == "hudson.model.Hudson.UploadPlugins" ||
             id == "hudson.model.Hudson.RunScripts" ||
             id == "hudson.model.Hudson.Administer") {
             // Do nothing
        } else {
            strategy.add(perm, team)
        }
    }
}
strategy.add(Jenkins.ADMINISTER, "admin")
// Uncomment below to see what webops see in production
// strategy.add(Jenkins.ADMINISTER, "{{jenkaas}.admin.user}")
strategy.add(Jenkins.ADMINISTER, "canonical-is")

strategy.add(Jenkins.READ, "anonymous")
strategy.add(Item.READ, "anonymous")
strategy.add(Computer.CONNECT, "anonymous")

for (build_bot in build_bots){
	strategy.add(Item.READ, build_bot)
	strategy.add(Item.BUILD, build_bot)
}

for (admin_bot in admin_bots){
    strategy.add(Item.READ, admin_bot)
    strategy.add(Item.CONFIGURE, admin_bot)
    strategy.add(Item.CREATE, admin_bot)
    strategy.add(Item.DELETE, admin_bot)
    // NO Item.BUILD
}

for (group in build_people){
	strategy.add(Item.READ, group)
	strategy.add(Item.BUILD, group)
}
 
instance.setAuthorizationStrategy(strategy)

//////////////////////////////////////////////////////////////////////
// Disable job execution on the master
//////////////////////////////////////////////////////////////////////
def why = new hudson.slaves.OfflineCause.ByCLI('No job execution allowed on master')
instance.toComputer().setTemporarilyOffline(true, why)
instance.setNumExecutors(0)


//////////////////////////////////////////////////////////////////////
// Save the 'admin' user's API token to disk for later usage.
//////////////////////////////////////////////////////////////////////

// Dump out the admin token
User u = User.get("admin")
ApiTokenProperty t = u.getProperty(ApiTokenProperty.class)
def token = t.getApiToken()
def token_file = new File('/var/lib/jenkins/.admin-api-token')
token_file.write "$token"


//////////////////////////////////////////////////////////////////////
// Create the slave
//////////////////////////////////////////////////////////////////////

def interfaces = NetworkInterface.getNetworkInterfaces()
def eth0_ip

/* Determine the IP address of the jenkins master */
for (iface in interfaces) {
  if (iface.name == "eth0") {
    def addresses = iface.inetAddresses
    iface.inetAddresses.each { addr ->
      if (addr.hostAddress.contains('.')) { // ipv4 addresses only
        eth0_ip = addr.hostAddress
      }
    }
  }
}

def nodes = hudson.model.Hudson.instance.nodes

for (node in nodes) {
    if (node.getNodeName() == slave_name) {
       println("'$slave_name' slave already exists, deleting")
       instance.removeNode(node)
    }
}


def description = "Slave " + slave_name
def slave_dir = "/var/lib/jenkins/"

instance.addNode(
    new DumbSlave(
      slave_name,
      description,
      slave_dir,
      // number of executors
      "1",
      // Use as much as possible
      Node.Mode.NORMAL,
      // use no labels by default until we have a need
      "",
      new JNLPLauncher("$eth0_ip:", ""),
      new RetentionStrategy.Always(),
      new LinkedList()
    )
  )
println("Added slave '$slave_name'")


instance.save()
